Imports System.IO
Imports System.Drawing
Imports System.Windows.Forms
Imports Microsoft.Win32

'
' A class to create a "watermark" on the User's existing wallpaper.  The
' watermark is temporary and is replaced upon next login.  It's designed to
' be subtle and not obnoxious.  It uses semi-transparent text on top of 
' whatever wallpaper the user already has.
'
Public Class Watermark

#Region "API Region"
    Private Const SPI_SETDESKWALLPAPER As Integer = &H14
    Private Const SPIF_UPDATEINIFILE As Integer = &H1
    Private Const SPIF_SENDWININICHANGE As Integer = &H2

    'BOOL SystemParametersInfo(
    '  UINT uiAction,
    '  UINT uiParam,
    '  PVOID pvParam,
    '  UINT fWinIni
    ');
    Private Declare Auto Function SystemParametersInfo Lib "user32.dll" ( _
        ByVal uAction As Integer, _
        ByVal uParam As Integer, _
        ByVal lpvParam As String, _
        ByVal fuWinIni As Integer _
    ) As Integer
#End Region

    Const X_Offset As Integer = 250
    Const Y_Offset As Integer = 5

    ' Note: There is no error checking here, so errors "bubble up" to
    ' the calling routine
    Public Sub MakeWallPaper(ByVal status As String)
        Dim x, y, wp_style, wp_tile As Integer
        Dim rect As Rectangle
        Dim hkcu, reg As RegistryKey
        Dim seed_file, bg_color, wp_file As String
        Dim bm_seed, bm_wp As Bitmap
        Dim gr As Graphics
        Dim br_text, br_shadow As SolidBrush
        Dim fnt As Font

        ' get the size of the desktop...
        rect = Screen.GetBounds(New Point(1, 1))
        x = rect.Width
        y = rect.Height

        ' get the current wallpaper settings from the registry
        hkcu = Registry.CurrentUser
        reg = hkcu.OpenSubKey("Control Panel\Desktop", False)
        seed_file = reg.GetValue("Wallpaper", "").ToString

        ' one more try...
        If Not File.Exists(seed_file) Then
            seed_file = reg.GetValue("OriginalWallpaper", "").ToString
        End If
        wp_style = CInt(reg.GetValue("WallpaperStyle", 0))
        wp_tile = CInt(reg.GetValue("TileWallpaper", 0))
        reg.Close()

        reg = hkcu.OpenSubKey("Control Panel\Colors", False)
        If IsNothing(reg) Then
            ' for Win9x, use the standard dark green
            bg_color = "0 128 128"
        Else
            bg_color = reg.GetValue("Background", "128 128 128").ToString
            reg.Close()
        End If

        ' load the bitmaps
        bm_seed = Nothing
        If seed_file = "" Or seed_file = "(None)" Then
            wp_style = -1
        Else
            If Not File.Exists(seed_file) Then
                Throw New ApplicationException("Error loading wallpaper seed file '" & seed_file & "'")
            End If
            bm_seed = New Bitmap(seed_file)
        End If

        ' create a new bitmap of the correct size
        bm_wp = New Bitmap(x, y)
        gr = Graphics.FromImage(bm_wp)

        ' let's figure out if we should center, tile or stretch.  I don't try to
        ' emulate the pattern feature Win9x/WinNT
        If wp_style = 0 Then
            wp_style += wp_tile
        End If
        Select Case wp_style
            Case -1 ' None
                Dim parts() As String
                Dim r, g, b As Integer
                Dim br As Brush

                parts = bg_color.Split(" "c)
                r = CInt(parts(0))
                g = CInt(parts(1))
                b = CInt(parts(2))
                br = New SolidBrush(System.Drawing.Color.FromArgb(r, g, b))
                gr.FillRectangle(br, 0, 0, x, y)
            Case 0 ' Center
                Dim parts() As String
                Dim r, g, b As Integer
                Dim br As Brush

                parts = bg_color.Split(" "c)
                r = CInt(parts(0))
                g = CInt(parts(1))
                b = CInt(parts(2))
                br = New SolidBrush(System.Drawing.Color.FromArgb(r, g, b))
                gr.FillRectangle(br, 0, 0, x, y)
                gr.DrawImage(bm_seed, (x - bm_seed.Width) \ 2, (y - bm_seed.Height) \ 2, bm_seed.Width, bm_seed.Height)
            Case 1 ' Tile
                Dim tile_x, tile_y As Integer

                ' I could do this with a graphics brush... but I chose to
                ' do multiple DrawImages instead.
                For tile_x = 0 To x Step bm_seed.Width
                    For tile_y = 0 To y Step bm_seed.Height
                        gr.DrawImage(bm_seed, tile_x, tile_y, bm_seed.Width, bm_seed.Height)
                    Next
                Next
            Case 2, 6, 10 ' Stretch
                ' TODO: Resize using Windows 7's new "6 & 10" options
                gr.DrawImage(bm_seed, 0, 0, x, y)
        End Select

        fnt = New Font("Arial", 10, FontStyle.Regular, GraphicsUnit.Pixel)

        ' Next we build the brushes we plan to use. 50% transparent black for the
        ' drop shadow and 50% transparent white for the text
        br_shadow = New SolidBrush(Color.FromArgb(128, 0, 0, 0))
        br_text = New SolidBrush(Color.FromArgb(128, 255, 255, 255))

        ' write the text (offset the drop shadow by 1 pixel)
        gr.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit
        gr.DrawString(status, fnt, br_shadow, (x - X_Offset) + 1, Y_Offset + 1)
        gr.DrawString(status, fnt, br_text, x - X_Offset, Y_Offset)
        gr.Save()

        ' save the resulting bitmap
        wp_file = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) & "\SOSOS.bmp"
        bm_wp.Save(wp_file, Imaging.ImageFormat.Bmp)

        ' clean up
        If wp_style >= 0 Then
            bm_seed.Dispose()
        End If
        bm_wp.Dispose()
        gr.Dispose()

        ' Set the wallpaper
        SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, wp_file, SPIF_SENDWININICHANGE Or SPIF_UPDATEINIFILE)

        ' Put the original settings back in the registry. The line above
        ' changes it to wp_file.  However, we want a "fresh" starting point
        ' each time a user log in, so we make it look like the wallpaper
        ' is the orginal seed_file (without the water mark).

        ' Note: this is the only place in the entire application that requires
        ' write permission to the registry
        reg = hkcu.OpenSubKey("Control Panel\Desktop", True)
        reg.SetValue("Wallpaper", seed_file)
        reg.Close()
    End Sub


End Class
